//---------------------------------------------------
// module  : f_DisplayHyper.cc                       
// auteur  : Max Mignotte                            
// date    :                                              
// langage : C++                                      
// labo    :                                  
//---------------------------------------------------

//-----------------------------------------------
// FICHIERS INCLUS ------------------------------
//-----------------------------------------------
#include "DisplayHyper.h"

//------------------------------------------------
// METHODES DE LA CLASSE DISPLAYHYPER  -------------
//------------------------------------------------
//----------------------------------------------------------
// Constructeur de la classe DisplayHyper
//----------------------------------------------------------
DisplayHyper::DisplayHyper()           
 {
  //Initialisation
  depth=DEPTH;
  length=LENGTH;
  width=WIDTH;

  //Allocation Memoire Matrice  
  ImgCube=simatrix_allocate_3d(depth,length,width);

  ImgF1=fmatrix_allocate_3d(TROIS,length,width);
  ImgCoulf=fmatrix_allocate_3d(TROIS,length,width);
  ImgCoul=imatrix_allocate_3d(TROIS,length,width);
  ImgCoulLAB=imatrix_allocate_3d(TROIS,length,width);

  ImgNB=fmatrix_allocate_2d(length,width);

  Img1=fmatrix_allocate_2d(length,width);
  Img2=fmatrix_allocate_2d(length,width);
  Img3=fmatrix_allocate_2d(length,width);
  Img4=fmatrix_allocate_2d(length,width);
  Img5=fmatrix_allocate_2d(length,width); 
  Img6=fmatrix_allocate_2d(length,width); 
  Img7=fmatrix_allocate_2d(length,width);
  Img8=fmatrix_allocate_2d(length,width);
  Img9=fmatrix_allocate_2d(length,width);

  BandInfo=fmatrix_allocate_2d(depth,4);

  //Parametre
  Buse=BRuse=BGuse=BBuse=0;
  NbIter=NBITER;
  Correlation=0.0;

  //Flag
  flag_corr=0;
 }


//----------------------------------------------------------
// Destructeur de la classe DisplayHyper
//----------------------------------------------------------
DisplayHyper::~DisplayHyper()
{ 
  //Libere Memoire Matrice 
  if (ImgCube)    free_simatrix_3d(ImgCube,depth); 

  if (ImgF1)      free_fmatrix_3d(ImgF1,TROIS);
  if (ImgCoulf)   free_fmatrix_3d(ImgCoulf,TROIS);
  if (ImgCoul)    free_imatrix_3d(ImgCoul,TROIS);
  if (ImgCoulLAB) free_imatrix_3d(ImgCoulLAB,TROIS);
 
  if (ImgNB)      free_fmatrix_2d(ImgNB); 

  if (Img1)       free_fmatrix_2d(Img1); 
  if (Img2)       free_fmatrix_2d(Img2);
  if (Img3)       free_fmatrix_2d(Img3);
  if (Img4)       free_fmatrix_2d(Img4);
  if (Img5)       free_fmatrix_2d(Img5);
  if (Img6)       free_fmatrix_2d(Img6);
  if (Img7)       free_fmatrix_2d(Img7);
  if (Img8)       free_fmatrix_2d(Img8);
  if (Img9)       free_fmatrix_2d(Img9);

  if (BandInfo)   free_fmatrix_2d(BandInfo);
}

//------------//
//--METHODES--//
//------------//
//----------------------------------------------------------
// Descente Multiresolution  Gradient Conjugue 
// -------------------------------------------
//
//----------------------------------------------------------    
void DisplayHyper::Display()
{
 int NbCnx;
 int DeltaBand;
 int B1,B2,B3,B4,B5,B6;
 int lgthS16,wdthS16;
 int lgthS0,wdthS0;

 //>PourTests
 char MyBuff[100];
 strcpy(MyBuff,Extension);

 //>Parametres
 NbCnx=CARRE(SzW);

 DeltaBand=(int)((float)depth/3.0);
 B1=0;
 B2=B3=DeltaBand;
 B4=B5=B3+DeltaBand;
 B6=depth-1; 

 lgthS16=length/16;
 wdthS16=width/16;
 lgthS0=length;
 wdthS0=width;

 //>TimeObjet
 Chrono  chrono; chrono.reset();

 //>Information
 printf("\n");
 printf("\n Image: [%s]",Name_Img);
 printf(" [Ext:%s]",Extension);
 printf("  Size:[depth:%d | lgth:%d | wdth:%d]",depth,length,width);
 printf("\n ---------");
 printf("\n Bandes...........[%d-%d] [%d-%d] [%d-%d]",B1,B2,B3,B4,B5,B6);
 printf("\n ---------");
 printf("\n SzWindow ........[%d]",SzW);
 printf("\n StepGradient.....[%.0f*10^-6]",StGr*MILLION);
 printf("\n ---------");
 printf("\n Scale............[16]");
 printf("\n [lgth_r=%d  wdth_r=%d]",lgthS16,wdthS16);
 printf("\n NbIter...........[%dx4]",NbIter);
 printf("\n ---------");
 if (flag_corr)     printf("\n > Mesure Correlation");
 printf("\n");
 fflush(stdout);

 //>AllocationMemoire
 float***  ImgS16=fmatrix_allocate_3d(TROIS,lgthS16,wdthS16);
 float***  ImgS0=fmatrix_allocate_3d(TROIS,length,width);
 float***  Img=fmatrix_allocate_3d(TROIS,length,width);
 int*** ImgCoulS16=imatrix_allocate_3d(TROIS,lgthS16,wdthS16);
 int*** ImgCoulS0=imatrix_allocate_3d(TROIS,length,width);
 
 //===========================================
 // ALGO ===========================================
 //===========================================

 //>Chargement & Visu
 //==================
 printf("\n Chargement AVIRIS Img...");
 LoadImgHyperspectral(Name_Img,ImgCube,depth,length,width);
 TimeSec=chrono.elapsed(SECOND); chrono.print(); 

 //SELECTION BANDS
 //===============
 SelectInfoBands(ImgCube,depth,length,width,0,BandInfo,RSBMIN); 
 
 //=====
 // 16 |
 //=============================================
 //>ConjugateGradientDescente LAB Bandes [SCALE]
 //=========================================================================
 //>L_Band
 MultiResConjGradDesc(ImgCube,length,width,B1,B2,SzW,NbCnx,ImgS16[0],16,NbIter*4);
 BRuse=Buse;
 NrjImgRInit=NrjImgInit;
 NrjImgR=NrjImg;
 StabR=Stab;

 //>A_Band
 MultiResConjGradDesc(ImgCube,length,width,B3,B4,SzW,NbCnx,ImgS16[1],16,NbIter*4);
 BGuse=Buse;
 NrjImgGInit=NrjImgInit;
 NrjImgG=NrjImg;
 StabG=Stab;

 //>B_Band
 MultiResConjGradDesc(ImgCube,length,width,B5,B6,SzW,NbCnx,ImgS16[2],16,NbIter*4);
 BBuse=Buse;
 NrjImgBInit=NrjImgInit;
 NrjImgB=NrjImg;
 StabB=Stab;
 //==========================================================================

 //>DEBUG
 // if (0) { MakeColorImg(ImgCoulS16,ImgS16[0],ImgS16[1],ImgS16[2],lgthS16,wdthS16);
 //           Recal(ImgCoulS16,lgthS16,wdthS16);
 //           CopyMat(ImgF1,ImgCoulS16,lgthS16,wdthS16,TROIS);
 //           Convert_LAB_TO_RGB(ImgF1,lgthS16,wdthS16,ImgCoulf);
 //           SaveImagePpm("HypImgS16","","",ImgCoulf,lgthS16,wdthS16); }
 TimeSec=chrono.elapsed(SECOND); chrono.print(); 

 //>DUPLICATION [SCALE>SCALE-0]
 //===========================
 printf("\n >> DUPLICATION  [16 --> 0]");
 InterpolDuplicate(ImgCube,length,width,B1,B2,ImgS16[0],lgthS16,wdthS16,ImgS0[0],length,width,5,12.0); 
 InterpolDuplicate(ImgCube,length,width,B3,B4,ImgS16[1],lgthS16,wdthS16,ImgS0[1],length,width,5,12.0);
 InterpolDuplicate(ImgCube,length,width,B5,B6,ImgS16[2],lgthS16,wdthS16,ImgS0[2],length,width,5,12.0); 
 TimeSec=chrono.elapsed(SECOND); chrono.print();

 //>DEBUG
 // if (0) { MakeColorImg(ImgCoulS0,ImgS0[0],ImgS0[1],ImgS0[2],lgthS0,wdthS0);
 //           Recal(ImgCoulS0,lgthS0,wdthS0);
 //           CopyMat(ImgF1,ImgCoulS0,lgthS0,wdthS0,TROIS);
 //           Convert_LAB_TO_RGB(ImgF1,lgthS0,wdthS0,ImgCoulf);
 //           SaveImagePpm("HypImgSD0","","",ImgCoulf,lgthS0,wdthS0); }

 //>Parametres NbCnx/VitesseGradient
 NbCnx=(int)((float)(CARRE(SzW)*0.05)+0.49)+1;

 //====
 // 0 |
 //===============================================
 //>ConjugateGradientDescente LAB Bandes [Full Res]
 //=========================================================================
 //>L_Band
 MultiResConjGradDesc(ImgCube,length,width,B1,B2,SzW,NbCnx,ImgS0[0],1,NbIter,ImgS0[0]);
 TimeSec=chrono.elapsed(SECOND); chrono.print();  

 //>A_Band
 MultiResConjGradDesc(ImgCube,length,width,B3,B4,SzW,NbCnx,ImgS0[1],1,NbIter,ImgS0[1]);
 TimeSec=chrono.elapsed(SECOND); chrono.print(); 

 //>B_Band
 MultiResConjGradDesc(ImgCube,length,width,B5,B6,SzW,NbCnx,ImgS0[2],1,NbIter,ImgS0[2]);
 TimeSec=chrono.elapsed(SECOND); chrono.print(); 
 //==========================================================================

 //>DEBUG
 // if (0) { MakeColorImg(ImgCoulS0,ImgS0[0],ImgS0[1],ImgS0[2],lgthS0,wdthS0);
 //           Recal(ImgCoulS0,lgthS0,wdthS0);
 //           SaveImagePpm("HypImgS0","","",ImgCoulS0,lgthS0,wdthS0); }


 //ConversionFinale LAB->RGB
 //=========================
 Convert_LAB_TO_RGB(ImgS0,length,width,ImgF1);
 Recal(ImgS0,length,width);
 Recal(ImgF1,length,width);
 CopyMat(ImgCoulLAB,ImgS0,lgthS0,wdthS0,TROIS);
 CopyMat(ImgCoul,ImgF1,lgthS0,wdthS0,TROIS);

 //Validation Cor/Featuresep
 //=========================
 if (flag_corr)
    { Correlation=SAM_Valid(ImgF1,ImgCube,depth,length,width,&FeatSep); 
      printf("\n  >> Correlation=%.4f",Correlation); }
 
 //Info
 //====
 FillInfoProcess();
 printf("\n Pour Info:\n");
 printf(" > %s",InfoProcess);
 
 //Sauvegarde
 //==========
 if (flag_save)
    { printf("\n  Sauvegarde");
      fflush(stdout);
      SaveImagePpm("HypImg_LAB",Extension,InfoProcess,ImgCoulLAB,length,width);
      SaveImagePpm("HypImg_RBV",Extension,InfoProcess,ImgCoul,length,width); }
  
 //Liberation Memoire
 if (ImgS16)     free_fmatrix_3d(ImgS16,TROIS); 
 if (ImgS0)      free_fmatrix_3d(ImgS0,TROIS);
 if (Img)        free_fmatrix_3d(Img,TROIS);
 if (ImgCoulS16) free_imatrix_3d(ImgCoulS16,TROIS);
 if (ImgCoulS0)  free_imatrix_3d(ImgCoulS0,TROIS);

 //End
 TimeSec=chrono.elapsed(SECOND); chrono.print();
 }

