//---------------------------------------------------
// module  : f_DisplayHyper1.cc                       
// auteur  : Max Mignotte                            
// date    :                                              
// langage : C++                                      
// labo    :                                  
//---------------------------------------------------

//-----------------------------------------------
// FICHIERS INCLUS ------------------------------
//-----------------------------------------------
#include "DisplayHyper.h"

//-----------------------------------------------
//METHODES --------------------------------------
//-----------------------------------------------
//----------------------------------------------------------
// Descente Gradient Conjugue (norme L2)
// ------------------------------------
//
// x[n+1]=x[n]+StGr*d[n]
// 
//  d[n]=- Grad_J(x[n])
//       + <Grad_J(x[n])-Grad_J(x[n-1]),Grad_J(x[n])>/<Grad_J(x[n-1]),Grad_J(x[n-1])> d[n-1]
//
// Img,Img8   : Img Initiale :: Image_Nrj_init
// Img1       : x[n] puis ...x[n+1]
// Img2       : Grad_J(x[n])
// Img3       : Grad_J(x[n-1])
// Img4       : d[n]
// Img5       : d[n-1]
// Img9       : x[n+1]
// 
// Img7       : ImgInit
//
// Img6       : unused 
//
//----------------------------------------------------------    
void DisplayHyper::ConjGradDesc(float** Img,float** ImgRes,int lgth,int wdth,\
				short int*** NonLocGrph,int NbCnx,int NbIter)
{
 int k;
 float nrj,nrj_prec,nrj_init;
 float stabi;
 float Beta;

 //>Init
 CopyMat(Img1,Img,lgth,wdth);
 CopyMat(Img8,Img,lgth,wdth);
 CopyMat(Img7,Img,lgth,wdth);
 nrj=ComputeTotalVariationL2(Img8,lgth,wdth,NonLocGrph,NbCnx);
 nrj_prec=nrj;
 nrj_init=nrj;
 printf("\n\n    >  ConjGradDescente");  
 printf(" [MoyImg=%.1f]",ComputeMean(Img,lgth,wdth));  
 printf("\n    >  <NrjInit=%.3f>",nrj);
 fflush(stdout);
 NrjImgInit=nrj_init;
 Beta=0.0;
 stabi=BIGNUMBER;

 //>Flag_stop
 int FlagStop=0;
 

 //>Conjugate Gradient Descent
 //===========================
 for(k=0;((k<=NbIter)||(!FlagStop));k++)
   {
     //>Info
     if ((k<20)||(!(k%10))) printf("\n [%d]>  ",k); 
    
     // Img2 <-- Grad_J(x[k]) 
     NonLocalGGMRFDeriveL2(Img1,Img2,lgth,wdth,NonLocGrph,NbCnx);

     //-------------------------------------------
     // [k==0] SimpleGradient
     //-------------------------------------------
     //      > d[k] <-- Grad_J(x[k])    
     //      > x[k+1]=x[n]-StrGr*Grad_J(x[k]) 
     if (k==0) 
        { CopyMat(Img4,Img2,lgth,wdth);
          AddMatrix(Img9,Img1,-1.0*StGr,Img2,lgth,wdth); }

     //--------------------------------------------
     // [k!=0] Gradient Conjugue
     //-------------------------------------------- 
     //      > x[k+1]=x[k]-StGr*d[k]
     if (k) 
       {  Beta=ComputeBeta(Img2,Img3,lgth,wdth);
          MultMatrix(Img6,Beta,Img5,lgth,wdth);
          AddMatrix(Img4,Img2,-1.0,Img6,lgth,wdth);
          AddMatrix(Img9,Img1,-1.0*StGr,Img4,lgth,wdth); }

     //>Calcul Nrj & Stab(x[k],x[k-1])
     if ((k<20)||(!(k%10)))
       {  nrj_prec=nrj;
          nrj=ComputeTotalVariationL2(Img9,lgth,wdth,NonLocGrph,NbCnx);
          stabi=ComputeStabNorm2(Img9,Img1,lgth,wdth);
          printf("<Nrj=%.2f> <Stab=%.2f> <Beta=%.2f>",nrj,stabi*BILLION,Beta*MILLION);
          fflush(stdout); }

    //>RecordBest
    if ((nrj<nrj_prec)&&(nrj<nrj_init)) CopyMat(Img8,Img9,lgth,wdth);
 
    //>FlagStop
    if ((!(k%10))&&((stabi*BILLION)<10)) { FlagStop=1; printf("*"); }

    //>Verif si StGr trop grand
    if (k<30)
      if ((nrj>nrj_prec)||(nrj>nrj_init)||(isnan(nrj)))
	 { CopyMat(Img9,Img8,lgth,wdth);
           StGr/=2.0;
           InitMatrix(Img5,lgth,wdth);
           printf(" ! New StGr=%.1f*10^-6",StGr*MILLION);
           k--;
           fflush(stdout); }
     
    // x[k+1] --> x[k]  
    // Img3 <-- Grad_J(x[k-1]) 
    // Img5 <-- d[k-1]
    CopyMat(Img1,Img9,lgth,wdth);
    CopyMat(Img3,Img2,lgth,wdth);
    CopyMat(Img5,Img4,lgth,wdth);

    //End
    if ((k>2*NbIter)||((k>30)&&((stabi*BILLION)<0.01))) break;  

   }//FIN_ALGO======================================

 //Enregistrement Nrj_finale[Img9] & Stabilite[Img9]
 NrjImg=nrj;
 Stab=stabi*BILLION;
 printf("\n      >> [Nrj:%.5f][Stab:%.3f]",NrjImg,Stab);
 printf("\n      >> [MoyImg=%.1f]",ComputeMean(Img9,lgth,wdth)); 
 printf("\n      >> [DiffImg=%.1f]",ComputeDiff(Img9,Img7,lgth,wdth));  

 //Transfert>ImgRes
 printf("\n");
 CopyMat(ImgRes,Img9,lgth,wdth);
}

//--------------------------------
//INFO ---------------------------
//--------------------------------
//----------------------------------------------------------
// Info 
//----------------------------------------------------------    
void DisplayHyper::FillInfoProcess()
{
 char Buff[NCHAR];

 //>ImgName
 strcpy(InfoProcess,Name_Img);

 //>Method
 sprintf(Buff,"_M%d",Method);
 strcat(InfoProcess,Buff);

 //>SzWindow/%Conx
 sprintf(Buff,"_S%d",SzW);
 strcat(InfoProcess,Buff);
 
 //>Nrj
 sprintf(Buff,"_Er_%.0f_%.0f",NrjImgRInit*100,NrjImgR*100);
 strcat(InfoProcess,Buff);
 sprintf(Buff,"_Eg_%.0f_%.0f",NrjImgGInit*100,NrjImgG*100);
 strcat(InfoProcess,Buff);
 sprintf(Buff,"_Eb_%.0f_%.0f",NrjImgBInit*100,NrjImgB*100);
 strcat(InfoProcess,Buff);

 //>Stab
 sprintf(Buff,"_Stab_%.0f_%.0f_%.0f",StabR,StabG,StabB);
 strcat(InfoProcess,Buff);

 //>Time
 sprintf(Buff,"_Sec%d",TimeSec);
 strcat(InfoProcess,Buff);
 
 //>Correlation
 sprintf(Buff,"_Cor%.3f",Correlation);
 strcat(InfoProcess,Buff);

 //>FeatureSeparability
 sprintf(Buff,"_F%.3f",FeatSep);
 strcat(InfoProcess,Buff);

 //>BandUse
 sprintf(Buff,"_B%d_%d_%d",BRuse,BGuse,BBuse);
 strcat(InfoProcess,Buff);

 //>Iterations
 sprintf(Buff,"_NbIt%d",NbIter);
 strcat(InfoProcess,Buff);
}

//--------------------------------
//MULTIRESOLUTION ----------------
//--------------------------------
//----------------------------------------------------------
// Multiresolution Descente Gradient Conjugue (norme L2)
//      <SansIngInit>
//----------------------------------------------------------    
void DisplayHyper::MultiResConjGradDesc(short int*** HypImg,int lgth,int wdth,int dpthBeg,int dpthEnd,\
int SzW,int NbCnx,float** ImgRes,int Scale,int NbIter)
{
 int wdth_r,lgth_r;

 //>Parametre
 lgth_r=(int)(lgth/Scale);
 wdth_r=(int)(wdth/Scale);

  //>Presentation
  printf("\n\n ____MultiResolution Conjugate Gradient____");
  printf("\n __________________________________________");
  printf("\n\n  >> Img   [Lgth=%d::Wdth=%d]",lgth,wdth);
  printf("\n  >> Scale [%d] [LgthRed=%d::WdthRed=%d]",Scale,lgth_r,wdth_r);
  printf("\n  >> NbCnx [%d]",NbCnx);
  fflush(stdout);

  //>AllocationMemoire
  short int***  NonLocGrphRed=simatrix_allocate_3d(TROIS,NbCnx,(wdth_r*lgth_r));
  float** ImgRed=fmatrix_allocate_2d(lgth_r,wdth_r);
  
  //>NonLocalMultiResGraph
  ComputeMultiresNonLocalGraphWindow(HypImg,dpthBeg,dpthEnd,lgth_r,wdth_r,NonLocGrphRed,SzW,NbCnx,Scale);
 
  //>ChooseInitBand & Multiresolution
  ChooseMultiResInitMatrixAmongBand(HypImg,lgth_r,wdth_r,dpthBeg,dpthEnd,ImgRed,NonLocGrphRed,NbCnx,&Buse,Scale);
     
  //>ConjugateGradientDecente ImgRed>ImgRes
  ConjGradDesc(ImgRed,ImgRes,lgth_r,wdth_r,NonLocGrphRed,NbCnx,NbIter);   

 //Liberation Memoire
 if (NonLocGrphRed) free_simatrix_3d(NonLocGrphRed,TROIS);
 if (ImgRed)        free_fmatrix_2d(ImgRed);
}

//----------------------------------------------------------
// Multiresolution Descente Gradient Conjugue (norme L2)
//      <Avec ImgInit>
//----------------------------------------------------------    
void DisplayHyper::MultiResConjGradDesc(short int*** HypImg,int lgth,int wdth,int dpthBeg,int dpthEnd,\
int SzW,int NbCnx,float** ImgRes,int Scale,int NbIter,float** ImgInit)
{
 int wdth_r,lgth_r;

 //>Parametre
 lgth_r=(int)(lgth/Scale);
 wdth_r=(int)(wdth/Scale);

  //>Presentation
  printf("\n\n ____MultiResolution Conjugate Gradient____");
  printf("\n ___________________________________________");
  printf("\n\n  >> Img   [Lgth=%d::Wdth=%d]",lgth,wdth);
  printf("\n  >> Scale [%d] [LgthRed=%d::WdthRed=%d]",Scale,lgth_r,wdth_r);
  printf("\n  >> NbCnx [%d]",NbCnx);
  printf("\n  >> Size  [%d]",SzW);
  fflush(stdout);

  //>AllocationMemoire
  short int***  NonLocGrphRed=simatrix_allocate_3d(TROIS,NbCnx,(wdth_r*lgth_r)); 
  float** ImgRedDebug=fmatrix_allocate_2d(lgth_r,wdth_r);
  
  //>NonLocalMultiResGraph
  ComputeMultiresNonLocalGraphWindow(HypImg,dpthBeg,dpthEnd,lgth_r,wdth_r,NonLocGrphRed,SzW,NbCnx,Scale);
     
  //>ConjugateGradientDecente ImgRed>ImgRes
  ConjGradDesc(ImgInit,ImgRes,lgth_r,wdth_r,NonLocGrphRed,NbCnx,NbIter);   

 //Liberation Memoire
 if (NonLocGrphRed) free_simatrix_3d(NonLocGrphRed,TROIS);
 if (ImgRedDebug)   free_fmatrix_2d(ImgRedDebug);
}

